import unittest
import torch
import torch.nn as nn
import torch.nn.functional as F
import numpy as np
from zetane import TorchViz
from zetane import context as ztn


class Net(nn.Module):
    def __init__(self):
        super(Net, self).__init__()
        self.conv1 = nn.Conv2d(1, 32, 3, 1)
        self.conv2 = nn.Conv2d(32, 64, 3, 1)
        self.conv3 = nn.Conv2d(64, 64, 3, 1)
        self.conv4 = nn.Conv2d(64, 16, 3, 1)
        self.dropout = nn.Dropout(0.5)
        self.fc = nn.Linear(1024, 10)
        self.pool = nn.MaxPool2d(2, 2)

    def forward(self, x):
        x = self.conv1(x)
        x = F.relu(x)
        x = self.conv2(x)
        x = F.relu(x)
        x = self.pool(x)
        x = self.conv3(x)
        x = F.relu(x)
        x = self.conv4(x)
        x = F.relu(x)
        x = torch.flatten(x, 1)
        x = self.dropout(x)
        x = self.fc(x)
        output = F.log_softmax(x, dim=1)
        return output


class TestTorch(unittest.TestCase):
    def setUp(self):
        self.viz = TorchViz()
        self.engine = ztn.Context()

        self.net = Net()
        self.random_input = np.random.randn(1, 1, 28, 28).astype('float32')
        self.t = torch.from_numpy(self.random_input)
        self.expected = self.net(self.t)

    def test_viz(self):
        self.viz.create_torch_model(self.net, self.t)
        self.viz.model_debug(self.t)

        output_data = self.engine.form('graph.nodes.node=LogSoftmax.outputs.output.tensor.data').update()
        output_shape = self.engine.form('graph.nodes.node=LogSoftmax.outputs.output.tensor.shape').update()

        self.assertTrue(output_data.has_received())
        self.assertTrue(output_shape.has_received())
        data_vals = output_data.get_values()
        shape_vals = output_shape.get_values()

        # Note that ONNX is only giving us two digits of precision with torch!!
        # np.testing.assert_array_almost_equal(data_vals, self.expected.detach().numpy().flatten(), decimal=7)
