import unittest
import sys, os
from importlib import import_module
import argparse

sys.path.insert(0, os.path.abspath(
    os.path.join(
        os.path.dirname(__file__), '..')))

from zetane import context as ztn

parser = argparse.ArgumentParser(description='Run Zetane tests.')
parser.add_argument('-m', '--module',
                    action='store',
                    help='run a test at a specific module path: model.onnx.test_onnx')
parser.add_argument('-g', '--group',
                    action='store',
                    help='Run a test group by folder: model.keras, model.torch, model.tf, model.onnx render, utils, main')
parser.add_argument('-d', '--discover',
                    action='store',
                    help='Discover all tests')


class TestWrapper():
    def __init__(self):
        self.loader = unittest.TestLoader()
        self.suite = unittest.TestSuite()
        self.runner = unittest.TextTestRunner(verbosity=3)

    def add_test(self, module_path):
        test_module = import_module(module_path)
        loaded = self.loader.loadTestsFromModule(test_module)
        self.suite.addTests(loaded)

    def remove_test(self, start_dir):
        print(self.suite)

    def launch(self):
        self.engine = ztn.Context()

    def discover(self, start_dir='.'):
        self.suite = self.loader.discover(start_dir)

    def run(self):
        self.launch()
        result = self.runner.run(self.suite)

if  __name__ == '__main__':
    args = parser.parse_args()
    test = TestWrapper()

    if args.group:
        test.discover(args.group)

    if args.module:
        test.add_test(args.module)

    if args.discover:
        test.discover()

    test.run()
